// 云函数入口文件
const cloud = require('wx-server-sdk')

cloud.init({
  env: cloud.DYNAMIC_CURRENT_ENV
})
const db = cloud.database();

// 云函数入口函数
exports.main = async(event, context) => {
  switch (event.action) {
    case 'addCart':
      {
        return addCart(event)
      }
    case 'updateCart':
      {
        return updateCart(event)
      }
    case 'deleteCartOne':
      {
        return deleteCartOne(event)
      }

    case 'deleteAllCart':
      {
        return deleteAllCart(event)
      }
    case 'findAll':
      {
        return findAll(event)
      }
    default:
      {
        return
      }
  }
}

/**
 * 购物车信息
 * 购物车Id,_openid,cart_count,cart_price,foodList
 */


/**
 * 添加购物车
 * 参数 食品的id、添加的食品数量
 * 1、查询购物车信息
 * 2、根据食品id查询食品对象
 * 3、根据新添加的食品数量 和单价，和购物车信息计算出新的购物车信息并保存在数据库
 */
async function addCart(event) {
  const {
    OPENID
  } = cloud.getWXContext();
  const cartResult = await findAll();
  const food = await getFood(event);
  const newCart = {};
  let isAdd = true;
  //更新购物车中食品信息
  console.log(cartResult);
  if (cartResult.data && cartResult.data.length !== 0) { // 如果查询出购物车信息
    const foodList = cartResult.data[0].foodList; // 查询购物车中食品信息
    if (foodList && foodList.length !== 0) { // 如果购物车中有食品信息
      foodList.push({ ...food,
        out_count: event.out_count
      }); // 将当前食品添加到购物车中，并更新out_put属性
      newCart.foodList = foodList;
    }

  } else {
    isAdd = true;
    newCart.foodList = [{ ...food,
      out_count: event.out_count
    }];
  }

  newCart.cart_count = newCart.foodList.reduce((total, num) => {
    return total + num.out_count
  }, 0)
  newCart.cart_price = newCart.foodList.reduce((total, num) => {
    return total + num.price * num.out_count
  }, 0)


  try {
    const result = await db.collection('cart').doc(String(OPENID)).set({
      data: {
        "_openid": OPENID,
        ...newCart
      }
    })
    if (result.errMsg == 'document.set:ok') {
      return {
        code: 1,
        data: null,
        msg: '添加成功'
      }
    }
    return {
      code: 1,
      data: null,
      msg: '添加失败'
    }
  } catch (e) {
    return {
      code: 1,
      data: null,
      msg: '添加失败'
    }
  }
}

/**
 * 更新购物车，参数包含购物车cartId,食品foodId,out_count
 */
async function updateCart(event) {
  // 1、根据id查询出购物车信息
  let cart = await db.collection('cart').doc(event.cartId).get();
  let newCart = {};
  newCart.foodList = cart.data.foodList.map(val => {
    if (val._id == event.foodId) {
      val.out_count = event.out_count;
    }
    return val;
  })

  newCart.cart_count = newCart.foodList.reduce((total, num) => {
    return total + num.out_count
  }, 0)
  newCart.cart_price = newCart.foodList.reduce((total, num) => {
    return total + num.price * num.out_count
  }, 0)

  try {
    const result = await db.collection('cart').doc(event.cartId).update({
      data: {
        ...newCart
      }
    });
    if (result.errMsg == 'document.update:ok' && result.stats.updated == 1) {
      return {
        code: 1,
        data: null,
        msg: '更新成功'
      }
    }
    return {
      code: 0,
      data: null,
      msg: '更新失败'
    }
  } catch (e) {
    return {
      code: 0,
      data: null,
      msg: '更新失败'
    }
  }
}
/**
 * 删除一个食品
 * 参数购物车cartId,食品foodId，当购物车中foodList 为空的时候，删除整个购物车
 */
async function deleteCartOne(event) {
  try {
    // 先匹配购物车中foodlist数量,如果foodlist有且只有一条信息，直接删除整个购物车，否则只删除其中一个食品信息
    let cart = await db.collection('cart').doc(event.cartId).get();
    let newFoodList = cart.data.foodList.filter(val => val._id !== event.foodId);
    let result = null;
    console.log(newFoodList);
    if (newFoodList.length == 0) {
      result = await db.collection('cart').doc(event.cartId).remove();
    } else {
      // 更新购物车信息
      let newCart = {};
      newCart.foodList = newFoodList;
      newCart.cart_count = newCart.foodList.reduce((total, num) => {
        return total + num.out_count
      }, 0)
      newCart.cart_price = newCart.foodList.reduce((total, num) => {
        return total + num.price * num.out_count
      }, 0)
      result = await db.collection('cart').doc(event.cartId).update({
        data: {
          ...newCart
        }
      })
    }
    console.log(result);
    if (result.errMsg == 'document.remove:ok' && result.stats.removed == 1) {
      return {
        code: 1,
        data: null,
        msg: '删除成功'
      }
    }
    if (result.errMsg == 'document.update:ok' && result.stats.updated == 1) {
      return {
        code: 1,
        data: null,
        msg: '删除成功'
      }
    }
    return {
      code: 0,
      data: null,
      msg: '删除失败'
    }
  } catch (e) {
    return {
      code: 0,
      data: null,
      msg: '删除失败'
    }
  }
}


async function deleteAllCart(event) {
  try {
    const result = await db.collection('cart').doc(event.cartId).remove();
    console.log(result);
    if (result.errMsg == 'document.remove:ok' && result.stats.removed > 0) {
      return {
        code: 1,
        data: null,
        msg: '删除成功'
      }
    }
    return {
      code: 0,
      data: null,
      msg: '删除失败'
    }
  } catch (e) {
    return {
      code: 0,
      data: null,
      msg: '删除失败'
    }
  }
}


/**
 * 查询出购物车信息
 */
async function findAll() {
  const {
    OPENID
  } = cloud.getWXContext();
  try {
    const result = await db.collection('cart').where({
      "_openid": OPENID
    }).get();
    console.log(result);
    if (result.errMsg == 'collection.get:ok') {
      return {
        code: 1,
        data: result.data,
        msg: '查询成功'
      }
    }
    return {
      code: 0,
      data: null,
      msg: '查询失败'
    }
  } catch (e) {
    return {
      code: 0,
      data: null,
      msg: '查询失败'
    }
  }
}

/**
 * 调用foodlist云函数 查询出当前食品对象
 */
async function getFood(event) {
  const food = await cloud.callFunction({
    name: 'foodList',
    data: {
      action: 'find',
      "_id": event.foodId
    }
  })
  console.log(food);
  return food.result.data;
}